/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "u2d_GL.h"
#include "u2d_core.h"

void u2d_GL::draw() {
  if(!valid()) {
    /*projection
     *viewport
     *  w() and h() are the window dimenstions
     *  valid() is turned on after FLTK draw() returns
     */
    /*
    ortho();
    */
    glViewport(5, 5, this->w() - 10, this->h() - 10);
  }
  /* draw */
  MyInit();
  drawgeometry();
  //  swap_buffers();
}

int u2d_GL::handle(int event) {
/* don't call any OpenGL draw function from inside handle() */
/* we handle no events for now */
  switch(event) {
    /*  case fltk::PUSH:

    return 1;
  case fltk::DRAG:

    return 1;

  case fltk::RELEASE:

    return 1;
    */
    /* those are events i want to to handle */
  default:
    /* the base class here handle the others */
    return fltk::GlWindow::handle(event);
  }
}

void u2d_GL::drawprimitive(Primitive *curprim) {

  /* Crappy colorization */
  /*
  if(curprim->object == 1) glColor3f(1.0, 1.0, 1.0);
  else if(curprim->object == 2) glColor3f(0.5, 0.5, 0.5);
  else if(curprim->object == 3) glColor3f(1.0, 0.0, 0.0);
  else if(curprim->object == 4) glColor3f(0.3, 0.7, 1.0);
  else if(curprim->object == 5) glColor3f(1.0, 0.0, 0.0);
  else if(curprim->object == 6) glColor3f(0.0, 1.0, 0.0);
  else if(curprim->object == 7) glColor3f(0.0, 0.0, 1.0);
  else if(curprim->object == 8) glColor3f(1.0, 1.0, 0.0);
  else if(curprim->object == 9) glColor3f(0.0, 1.0, 1.0);
  else if(curprim->object == 10) glColor3f(1.0, 0.0, 1.0);
  */

  switch(curprim->type) {
    case PR_LINE:
#ifdef DEBUGGING
      fprintf(stdout, "x1: %.2f y1: %.2f, x2: %.2f y2: %.2f\n", \
	      ((Line *)curprim)->x1, ((Line *)curprim)->y1, \
	      ((Line *)curprim)->x2, ((Line *)curprim)->y2);
#endif
      glBegin(GL_LINES);
      glVertex2d(((Line *)curprim)->x1,((Line *)curprim)->y1);
      glVertex2d(((Line *)curprim)->x2,((Line *)curprim)->y2);
      glEnd();
      glFlush();
      break;
    case PR_CIRCLE:
      if(((Circle *)curprim)->startarc < ((Circle *)curprim)->endarc){
	/* Start and end arc are _not_ the same, start arc is _smaller_ than end arc */
	glBegin(GL_LINE_STRIP);
	for(int i = 0; i < 200; i++){
#ifdef DEBUGGING
	  printf("Current angle: %f Start arc: %f End arc %f ", \
		 2.0 * PI * i / 200.0 + ((Circle *)curprim)->startarc / 360.0 * PI * 2.0, \
		 ((Circle *)curprim)->startarc / 360.0 * PI * 2.0, \
		 ((Circle *)curprim)->endarc / 360.0 * PI * 2.0);
#endif
	  if((2.0 * PI * i / 200.0 + ((Circle *)curprim)->startarc / 360.0 * PI * 2.0 <= \
	      ((Circle *)curprim)->endarc / 360.0 * PI * 2.0)){
#ifdef DEBUGGING
	    printf("Drawn!");
#endif
	    glVertex2d(((Circle *)curprim)->centerx + ((Circle *)curprim)->radius * \
		       cos(2.0 * PI * i / 200.0 + ((Circle *)curprim)->startarc / 360.0 * PI * 2.0), \
		       ((Circle *)curprim)->centery + ((Circle *)curprim)->radius * \
		       sin(2.0 * PI * i / 200.0 + ((Circle *)curprim)->startarc / 360.0 * PI * 2.0));
	  }
#ifdef DEBUGGING
	  printf("\n");
#endif
	}
      }
      else if(((Circle *)curprim)->startarc >= ((Circle *)curprim)->endarc){
	/* Start and end arc are the same or start arc is _greater_ than end arc */
	int iter; /* Uggly hack to turn around the limit of 120-125 points of GL_LINE_LOOP */
	if(isnear(DEFAULT_TOLL, ((Circle *)curprim)->startarc, ((Circle *)curprim)->endarc)){
	  printf("closed circle\n");
	  glBegin(GL_LINE_LOOP);
	  iter = 120;
	}
	else{
	  glBegin(GL_LINE_STRIP);
	  iter = 200;
	}
	for(int i = 0; i < iter; i++){
#ifdef DEBUGGING
	  printf("Current angle: %f Start arc: %f End arc %f ", \
		 2.0 * PI * i / (double)iter - (360 - ((Circle *)curprim)->startarc) / 360.0 * PI * 2.0, \
		 ((Circle *)curprim)->startarc / 360.0 * PI * 2.0, \
		 ((Circle *)curprim)->endarc / 360.0 * PI * 2.0);
#endif
	  if((2.0 * PI * i / (double)iter - (360 - ((Circle *)curprim)->startarc) / 360.0 * PI * 2.0 <= \
	      ((Circle *)curprim)->endarc / 360.0 * PI * 2.0)){
#ifdef DEBUGGING
	    printf("Drawn!");
#endif
	    /* to print the coordinates that give the bug with full circles*/
	    /*
	    printf("%f %f\n",\
		   ((Circle *)curprim)->centerx + ((Circle *)curprim)->radius * \
		   cos(2.0 * PI * i / (double)iter - (360 - ((Circle *)curprim)->startarc) / 360.0 * PI * 2.0), \
		   ((Circle *)curprim)->centery + ((Circle *)curprim)->radius * \
		   sin(2.0 * PI * i / (double)iter - (360 - ((Circle *)curprim)->startarc) / 360.0 * PI * 2.0));
	    */
	    glVertex2d(((Circle *)curprim)->centerx + ((Circle *)curprim)->radius * \
		       cos(2.0 * PI * i / (double)iter - (360 - ((Circle *)curprim)->startarc) / 360.0 * PI * 2.0), \
		       ((Circle *)curprim)->centery + ((Circle *)curprim)->radius * \
		       sin(2.0 * PI * i / (double)iter - (360 - ((Circle *)curprim)->startarc) / 360.0 * PI * 2.0));
	  }
#ifdef DEBUGGING
	  printf("\n");
#endif
	}
      }

      /* Temporary fix to connect starting and ending point */
      /* Too much temporary! It does not allready anymore work now that I enabled arc drawing */
      /* This fix should not be needed anymore.
       * It is real old now, because I changed the arc drawing algo.
       *
       * if(isnear(DEFAULT_TOLL, ((Circle *)curprim)->startarc, ((Circle *)curprim)->endarc)){
       *	glVertex2f(((Circle *)curprim)->centerx + \
       *		   ((Circle *)curprim)->radius * cos(((Circle *)curprim)->startarc / 360.0 * PI * 2.0), \
       *                   ((Circle *)curprim)->centery + \
       *		   ((Circle *)curprim)->radius * sin(((Circle *)curprim)->endarc / 360.0 * PI * 2.0));
       */
      glEnd();
      glFlush();
      break;
    case PR_ELLIPSE:
      if(((Ellipse *)curprim)->startarc < ((Ellipse *)curprim)->endarc){
	/* Start and end arc are _not_ the same, start arc is _smaller_ than end arc */
	glBegin(GL_LINE_STRIP);
	for(int i = 0; i < 200; i++){
	  if((2.0 * PI * i / 200.0 + ((Ellipse *)curprim)->startarc / 360.0 * PI * 2.0 <= \
	      ((Ellipse *)curprim)->endarc / 360.0 * PI * 2.0)){
	    glVertex2d(((Ellipse *)curprim)->centerx + ((Ellipse *)curprim)->semix * \
		       cos(2.0 * PI * i / 200.0 + ((Ellipse *)curprim)->startarc / 360.0 * PI * 2.0), \
		       ((Ellipse *)curprim)->centery + ((Ellipse *)curprim)->semiy * \
		       sin(2.0 * PI * i / 200.0 + ((Ellipse *)curprim)->startarc / 360.0 * PI * 2.0));
	  }
	}
      }
      else if(((Ellipse *)curprim)->startarc >= ((Ellipse *)curprim)->endarc){
	/* Start and end arc are the same or start arc is _greater_ than end arc */
	int iter; /* Uggly hack to turn around the limit of 120-125 points of GL_LINE_LOOP */
	if(isnear(DEFAULT_TOLL, ((Ellipse *)curprim)->startarc, ((Ellipse *)curprim)->endarc)){
	  glBegin(GL_LINE_LOOP);
	  iter = 120;
	}
	else{
	  glBegin(GL_LINE_STRIP);
	  iter = 200;
	}
	for(int i = 0; i < iter; i++){
	  if((2.0 * PI * i / (double)iter - (360 - ((Ellipse *)curprim)->startarc) / 360.0 * PI * 2.0 <= \
	      ((Ellipse *)curprim)->endarc / 360.0 * PI * 2.0)){
	    glVertex2d(((Ellipse *)curprim)->centerx + ((Ellipse *)curprim)->semix * \
		       cos(2.0 * PI * i / (double)iter - (360 - ((Ellipse *)curprim)->startarc) / 360.0 * PI * 2.0),
		       ((Ellipse *)curprim)->centery + ((Ellipse *)curprim)->semiy * \
		       sin(2.0 * PI * i / (double)iter - (360 - ((Ellipse *)curprim)->startarc) / 360.0 * PI * 2.0));
	  }
	}
      }
      glEnd();
      glFlush();
      break;
    case PR_DATA:
      glBegin(GL_LINE_STRIP);
      for(int i = 1; i < (((Data *)curprim)->npoints - 1)*2; i = i + 2) {
	glVertex2d(((Data *)curprim)->points[i-1], ((Data *)curprim)->points[i]);
      }
      glVertex2d(((Data *)curprim)->points[((Data *)curprim)->npoints*2 - 1],\
		 ((Data *)curprim)->points[((Data *)curprim)->npoints*2]);
      glVertex2d(((Data *)curprim)->points[0], ((Data *)curprim)->points[1]);
      break;
    default:
      fprintf(stderr, "drawprimitive(): Unknown primitive type\n");
    }
}

void u2d_GL::drawgeometry() {
  //  make_current();
#ifdef DEBUGGING
  //  fprintf(stdout, "inside drawgeometry\n");
#endif
  glClear(GL_COLOR_BUFFER_BIT);
  Primitive *currentprimitive;
  currentprimitive = firstprimitive;
  for(int i = 1; i <= nprim; i++) {
    drawprimitive(currentprimitive);
    if(currentprimitive->nextprimitive != NULL) {
      currentprimitive = currentprimitive->nextprimitive;
    }
    else if (currentprimitive->nextprimitive == NULL && i != nprim) {
      fprintf(stderr, "drawgeometry: WTF!? How we are here?, nprim say we still have" \
                      "primitive to draw but currentprimitive->nextprimitive" \
                      "pointer is NULL. nprim MISMATCH!!!!\n");
      return;
    }
  }
}

void u2d_GL::set_ref_sizes(double minX, double maxX, double minY, double maxY){
  /* This is a fix for:
   * When autoscaling the gl window, and only a one dimensional element,
   * like an horizontal or vertical line, is given the scale can't be
   * guessed correctly, that's because we do not have a reference
   * dimension for both X and Y so one of those will be 0 actually
   * preventing correct drawing.
   */

  if((minX != 0 || maxX != 0) && (minY == 0 && maxY == 0)){
#ifdef DEBUGGING
    printf("X size overriding Y\n");
#endif
    minY = minX;
    maxY = maxX;
  }
  else if((minY != 0 || maxY != 0) && (minX == 0 && maxX == 0)){
#ifdef DEBUGGING
    printf("Y size overriding X\n");
#endif
    minX = maxY;
    maxX = maxY;
  }

  printf("ref_size sets: %g, %g, %g, %g\n", minX, maxX, minY, maxY);

  ref_minX = minX;
  ref_maxX = maxX;
  ref_minY = minY;
  ref_maxY = maxY;
}

/* guess_ref_size() is VERY cpu costly compared to
 * guess_ref_size(prim), call it with "drops counter"
 */
void u2d_GL::guess_ref_size(){

  double minX = 0;
  double maxX = 0;
  double minY = 0;
  double maxY = 0;

  Primitive *prim = firstprimitive;

  while(prim){
    switch(prim->type){
    case PR_LINE:
      minX = ((Line *)prim)->x1 < minX ? ((Line *)prim)->x1 : minX;
      maxX = ((Line *)prim)->x1 > maxX ? ((Line *)prim)->x1 : maxX;
      minX = ((Line *)prim)->x2 < minX ? ((Line *)prim)->x2 : minX;
      maxX = ((Line *)prim)->x2 > maxX ? ((Line *)prim)->x2 : maxX;

      minY = ((Line *)prim)->y1 < minY ? ((Line *)prim)->y1 : minY;
      maxY = ((Line *)prim)->y1 > maxY ? ((Line *)prim)->y1 : maxY;
      minY = ((Line *)prim)->y2 < minY ? ((Line *)prim)->y2 : minY;
      maxY = ((Line *)prim)->y2 > maxY ? ((Line *)prim)->y2 : maxY;
      break;

    case PR_CIRCLE:
      minX = (((Circle *)prim)->centerx - ((Circle *)prim)->radius) < minX ? \
             (((Circle *)prim)->centerx - ((Circle *)prim)->radius) : minX;
      maxX = (((Circle *)prim)->centerx + ((Circle *)prim)->radius) > maxX ? \
	     (((Circle *)prim)->centerx + ((Circle *)prim)->radius) : maxX;

      minY = (((Circle *)prim)->centery - ((Circle *)prim)->radius) < minY ? \
	     (((Circle *)prim)->centery - ((Circle *)prim)->radius) : minY;
      maxY = (((Circle *)prim)->centery + ((Circle *)prim)->radius) > maxY ? \
	     (((Circle *)prim)->centery + ((Circle *)prim)->radius) : maxY;
      break;

    case PR_ELLIPSE:
      minX = (((Ellipse *)prim)->centerx - ((Ellipse *)prim)->semix) < minX ? \
	     (((Ellipse *)prim)->centerx - ((Ellipse *)prim)->semix) : minX;
      maxX = (((Ellipse *)prim)->centerx + ((Ellipse *)prim)->semix) > maxX ? \
	     (((Ellipse *)prim)->centerx + ((Ellipse *)prim)->semix) : maxX;

      minY = (((Ellipse *)prim)->centery - ((Ellipse *)prim)->semiy) < minY ? \
	     (((Ellipse *)prim)->centery - ((Ellipse *)prim)->semiy) : minY;
      maxY = (((Ellipse *)prim)->centery + ((Ellipse *)prim)->semiy) > maxY ? \
	     (((Ellipse *)prim)->centery + ((Ellipse *)prim)->semiy) : maxY;
      break;

    case PR_DATA:
      for(int i = 1; i < (((Data *)prim)->npoints - 1)*2; i = i + 2) {
	minX = ((Data *)prim)->points[i - 1] < minX ? ((Data *)prim)->points[i - 1] : minX;
	maxX = ((Data *)prim)->points[i - 1] > maxX ? ((Data *)prim)->points[i - 1] : maxX;

	minY = ((Data *)prim)->points[i] < minY ? ((Data *)prim)->points[i] : minY;
	maxY = ((Data *)prim)->points[i] > maxY ? ((Data *)prim)->points[i] : maxY;
      }
      break;
    }
    prim = prim->nextprimitive;
  }
  set_ref_sizes(minX, maxX, minY, maxY);
}

void u2d_GL::guess_ref_size(Primitive *prim){

  double minX = ref_minX;
  double maxX = ref_maxX;
  double minY = ref_minY;
  double maxY = ref_maxY;

  switch(prim->type){
  case PR_LINE:
    minX = ((Line *)prim)->x1 < minX ? ((Line *)prim)->x1 : minX;
    maxX = ((Line *)prim)->x1 > maxX ? ((Line *)prim)->x1 : maxX;
    minX = ((Line *)prim)->x2 < minX ? ((Line *)prim)->x2 : minX;
    maxX = ((Line *)prim)->x2 > maxX ? ((Line *)prim)->x2 : maxX;

    minY = ((Line *)prim)->y1 < minY ? ((Line *)prim)->y1 : minY;
    maxY = ((Line *)prim)->y1 > maxY ? ((Line *)prim)->y1 : maxY;
    minY = ((Line *)prim)->y2 < minY ? ((Line *)prim)->y2 : minY;
    maxY = ((Line *)prim)->y2 > maxY ? ((Line *)prim)->y2 : maxY;
    break;

  case PR_CIRCLE:
    minX = (((Circle *)prim)->centerx - ((Circle *)prim)->radius) < minX ? \
           (((Circle *)prim)->centerx - ((Circle *)prim)->radius) : minX;
    maxX = (((Circle *)prim)->centerx + ((Circle *)prim)->radius) > maxX ? \
           (((Circle *)prim)->centerx + ((Circle *)prim)->radius) : maxX;

    minY = (((Circle *)prim)->centery - ((Circle *)prim)->radius) < minY ? \
           (((Circle *)prim)->centery - ((Circle *)prim)->radius) : minY;
    maxY = (((Circle *)prim)->centery + ((Circle *)prim)->radius) > maxY ? \
           (((Circle *)prim)->centery + ((Circle *)prim)->radius) : maxY;
    break;

  case PR_ELLIPSE:
    minX = (((Ellipse *)prim)->centerx - ((Ellipse *)prim)->semix) < minX ? \
           (((Ellipse *)prim)->centerx - ((Ellipse *)prim)->semix) : minX;
    maxX = (((Ellipse *)prim)->centerx + ((Ellipse *)prim)->semix) > maxX ? \
           (((Ellipse *)prim)->centerx + ((Ellipse *)prim)->semix) : maxX;

    minY = (((Ellipse *)prim)->centery - ((Ellipse *)prim)->semiy) < minY ? \
           (((Ellipse *)prim)->centery - ((Ellipse *)prim)->semiy) : minY;
    maxY = (((Ellipse *)prim)->centery + ((Ellipse *)prim)->semiy) > maxY ? \
           (((Ellipse *)prim)->centery + ((Ellipse *)prim)->semiy) : maxY;
    break;

  case PR_DATA:
    for(int i = 1; i < (((Data *)prim)->npoints - 1)*2; i = i + 2) {
      minX = ((Data *)prim)->points[i - 1] < minX ? ((Data *)prim)->points[i - 1] : minX;
      maxX = ((Data *)prim)->points[i - 1] > maxX ? ((Data *)prim)->points[i - 1] : maxX;

      minY = ((Data *)prim)->points[i] < minY ? ((Data *)prim)->points[i] : minY;
      maxY = ((Data *)prim)->points[i] > maxY ? ((Data *)prim)->points[i] : maxY;
    }
    break;
  }

  set_ref_sizes(minX, maxX, minY, maxY);
}

void u2d_GL::MyInit(){

  float factor;
  double view_minX;
  double view_maxX;
  double view_minY;
  double view_maxY;
  double center;
 
  /* This keep aspect ratio */

  double deltaY = ref_maxY - ref_minY;
  double deltaX = ref_maxX - ref_minX;
  if((deltaX / (double)this->w()) < (deltaY / (double)this->h())) {
    factor = (deltaY / (double)this->h()) / (deltaX / (double)this->w());
    center = (ref_maxX + ref_minX) / 2.0;
    view_minY = ref_minY;
    view_maxY = ref_maxY;
    view_minX = center - (center - ref_minX)*factor;
    view_maxX = center + (ref_maxX - center)*factor;
  }
  else {
    factor = (deltaX / (double)this->w()) / (deltaY / (double)this->h());
    center = (ref_maxY + ref_minY) / 2.0;
    view_minY = center - (center - ref_minY)*factor;
    view_maxY = center + (ref_maxY - center)*factor;
    view_minX = ref_minX;
    view_maxX = ref_maxX;
  }


  /* We take as reference the smaller area drawable */
  /* This algo does not keep aspect ratio */
  /*
  if(this->h() >= this->w()){
    factor = (double)this->h() / (double)this->w();
    center = (ref_maxY + ref_minY) / 2.0;
    view_minY = center - (center - ref_minY)*factor;
    view_maxY = center + (ref_maxY - center)*factor;
    view_minX = ref_minX;
    view_maxX = ref_maxX;
  }
  else{
    factor = (double)this->w() / (double)this->h();
    center = (ref_maxX + ref_minX) / 2.0;
    view_minY = ref_minY;
    view_maxY = ref_maxY;
    view_minX = center - (center - ref_minX)*factor;
    view_maxX = center + (ref_maxX - center)*factor;
  }
  */

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glOrtho(view_minX * 1.00, view_maxX * 1.00, view_minY * 1.00, view_maxY * 1.00, -1, 1 );

  /* Set up colors and line width */
  glClearColor(0.0, 0.0, 0.0, 0.0); //black
  glClear(GL_COLOR_BUFFER_BIT); // the background
  glColor3f(1.0, 1.0, 1.0);  //white
  glLineWidth((GLfloat)1); // 1 width for lines

  /* The next 3 lines make opengl lines anti-aliased and good looking,
   * removing one will not do the trick anymore, look like they
   * work togheder
   */
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void u2d_GL::set_prim_list_start(Primitive *prim, int n) {
  firstprimitive = prim;
  nprim = n;
}



/*

// Called when the window is resized
//		w, h - width and height of the window in pixels.
void resizeWindow(int w, int h)
{
	double scale, center;
	double windowXmin, windowXmax, windowYmin, windowYmax;

	// Define the portion of the window used for OpenGL rendering.
	glViewport( 0, 0, w, h );	// View port uses whole window

	// Set up the projection view matrix: orthographic projection
	// Determine the min and max values for x and y that should appear in the window.
	// The complication is that the aspect ratio of the window may not match the
	//		aspect ratio of the scene we want to view.
	w = (w==0) ? 1 : w;
	h = (h==0) ? 1 : h;
	if ( (Xmax-Xmin)/w < (Ymax-Ymin)/h ) {
		scale = ((Ymax-Ymin)/h)/((Xmax-Xmin)/w);
		center = (Xmax+Xmin)/2;
		windowXmin = center - (center-Xmin)*scale;
		windowXmax = center + (Xmax-center)*scale;
		windowYmin = Ymin;
		windowYmax = Ymax;
	}
	else {
		scale = ((Xmax-Xmin)/w)/((Ymax-Ymin)/h);
		center = (Ymax+Ymin)/2;
		windowYmin = center - (center-Ymin)*scale;
		windowYmax = center + (Ymax-center)*scale;
		windowXmin = Xmin;
		windowXmax = Xmax;
	}
	
	// Now that we know the max & min values for x & y that should be visible in the window,
	//		we set up the orthographic projection.
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrtho( windowXmin, windowXmax, windowYmin, windowYmax, -1, 1 );

}
*/
