/*   umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <fltk/xpmImage.h>
#include <fltk/Browser.h>
#include <fltk/MultiBrowser.h>
#include <fltk/run.h>
#include <fltk/Button.h>
#include <fltk/Window.h>
#include <fltk/Group.h>
#include <fltk/CheckButton.h>
#include <fltk/SharedImage.h>
#include <fltk/Output.h>
#include <fltk/Color.h>
//#include <fltk/Box.h>

#include "umesh2dgui.h"
#include "u2d_core.h"
#include "u2d_GL.h"
#include "BDFile.h"
#include "ascii_color.h"

#include "open_icon.xpm"
#include "save_7.xpm"
#include "cancel_15.xpm"
#include "create_12.xpm"


#define COLUMN1 50
#define COLUMN2 190
#define COLUMN3 310
#define COLUMN4 450

/* GLOBALS */
u2d_GL *u2d_gl;
BDFile *bdfile;
u2d_Core *u2d_core;
/* the 2 tabs */
fltk::Group *boundarygroup;
fltk::Group *domaingroup;
/* useful groups */
fltk::ScrollGroup *datagroup;
fltk::Group *subwindow;
/* the three windows */
fltk::Window *window;
fltk::Window *opengl_window;
fltk::Window *prim_browser_window;
fltk::MultiBrowser *prim_browser;
/* the message bar */
fltk::Output *msgout;
/* the primitive menubar items */
/*
fltk::Item *lineitem;
fltk::Item *circleitem;
fltk::Item *ellipseitem;
fltk::Item *dataitem;
*/
/* a terrible list of indexes for primitive values*/
int inputindex[10];
int filenameinput_i; //trailing _i is for index
int ydataposition = 0;
/* global icons */
fltk::xpmImage *cancel_icon;


int main(int argc, char ** argv){

  /* PRINT SOME ABOUT INFO TO STDOUT */
  fprintf(stdout, "%s%s%s by %sGabriele Dini Ciacci%s - version %s%s%s\n", \
  ASC_RED, PACKAGE, ASC_RESET, ASC_GREEN, ASC_RESET, ASC_BLUE, VERSION, ASC_RESET);

  /* PARSE THE ARGUMENTS WITH FLTK INTERNAL FUNCTION */
  fltk::args(argc, argv);

  /* CLASS DECLARATION */
  bdfile = new BDFile;
  u2d_core = new u2d_Core;
  window = new fltk::Window(640, 530, "umesh2dgui");
  opengl_window = new fltk::Window(202, 202, "OpenGL preview");
  prim_browser_window = new fltk::Window(640, 240, "Primitive browser");

  /* ICON IMAGE DECLATATION */
  fltk::xpmImage *open_icon;
  open_icon = new fltk::xpmImage(open_icon_xpm);
  fltk::xpmImage *save_icon;
  save_icon = new fltk::xpmImage(save_7_xpm);
  fltk::xpmImage *create_icon;
  create_icon = new fltk::xpmImage(create_12_xpm);
  cancel_icon = new fltk::xpmImage(cancel_15_xpm);

  /* WINDOW SETUP */
  window->callback((fltk::Callback *)quititem_cb);
  window->begin();
  fltk::MenuBar *menubar = new fltk::MenuBar(0, 0, 640, 30, 0);
  fltk::TabGroup *tabbar = new fltk::TabGroup(10, 30, 620, 480, 0);
  msgout = new fltk::Output(0, 510, 640, 20);
  msgout->color(fltk::GRAY75);
  msgout->value("Select a primitive to start");
  window->end();

  /* OPENGL WINDOW SETUP*/
  opengl_window->begin();
  u2d_gl = new u2d_GL::u2d_GL(1, 1, 200, 200, 0);
  opengl_window->end();
  opengl_window->resizable(u2d_gl);

  /* PRIMITIVE BROWSER WINDOW SETUP */
  const char *prim_browser_labels[] = {
    "Id",
    "Primitive Name",
    "Obj",
    "Type",
    "x1",
    "y1",
    "x2",
    "y2",
    "radius",
    "S arc",
    "E arc",
    0
  };
  int prim_browser_widths[] = {
    20,
    -1,
    20,
    50,
    70,
    70,
    70,
    70,
    70,
    50,
    50,
    0
  };
  prim_browser_window->begin();
  prim_browser = new fltk::MultiBrowser(0, 0, prim_browser_window->w(), \
					prim_browser_window->h() - 45, \
					"Primitive List");
  prim_browser->column_widths(prim_browser_widths);
  prim_browser->column_labels(prim_browser_labels);

  fltk::Button *browser_del_prim_button = new fltk::Button(10, prim_browser_window->h() - 35, 70, 25, "Delete");
  browser_del_prim_button->callback((fltk::Callback *)browser_del_prim_cb);
  /* Name button */
  /*
  fltk::Button *prim_list_name;
  prim_list_name = new fltk::Button(1, 1, 200, 20, "Primitive Name");
  prim_list_name->align(fltk::ALIGN_LEFT | fltk::ALIGN_INSIDE);
  */
  /* Type button */
  /*
  fltk::Button *prim_list_type;
  prim_list_type = new fltk::Button(200, 1, 50, 20, "Type");
  prim_list_type->align(fltk::ALIGN_LEFT | fltk::ALIGN_INSIDE);
  */
  /* x1 button */
  /*
  fltk::Button *prim_list_x1;
  prim_list_x1 = new fltk::Button(250, 1, 50, 20, "x1 coord");
  prim_list_x1->align(fltk::ALIGN_LEFT | fltk::ALIGN_INSIDE);
  */
  /* y1 button */
  /*
  fltk::Button *prim_list_y1;
  prim_list_y1 = new fltk::Button(300, 1, 50, 20, "y1 coord");
  prim_list_y1->align(fltk::ALIGN_LEFT | fltk::ALIGN_INSIDE);
  */
  /* x2 button */
  /*
  fltk::Button *prim_list_x2;
  prim_list_x2 = new fltk::Button(350, 1, 50, 20, "x2 coord");
  prim_list_x2->align(fltk::ALIGN_LEFT | fltk::ALIGN_INSIDE);
  */
  /* y2 button */
  /*
  fltk::Button *prim_list_y2;
  prim_list_y2 = new fltk::Button(400, 1, 50, 20, "y2 coord");
  prim_list_y2->align(fltk::ALIGN_LEFT | fltk::ALIGN_INSIDE);
  */

  prim_browser_window->end();

  /* TAB BAR SETUP */
  //  tabbar->box(fltk::UP_BOX);
  //  tabbar->box(fltk::BORDER_BOX);
  tabbar->box(fltk::THIN_UP_BOX);
  tabbar->begin(); // 620 450
  boundarygroup = new fltk::Group(0, 30, 620, 440, "Boundary");
  boundarygroup->box(fltk::ENGRAVED_BOX);
  domaingroup =  new fltk::Group(0, 30, 620, 440, "Domain");
  domaingroup->box(fltk::EMBOSSED_BOX);
  tabbar->end();

  /* TITLE BOX SETUP */ /* removed now */
  /*  fltk::Widget *box = new fltk::Widget(10, 30, 620, 40, "umesh2dgui");
   *  box->box(fltk::UP_BOX);
   *  box->labelfont(fltk::HELVETICA_BOLD_ITALIC);
   *  box->labelcolor(fltk::GREEN);
   *  box->labelsize(40);
   *  box->labeltype(fltk::SHADOW_LABEL);
   */

  /* BOUNDARY TAB */
  boundarygroup->begin();

  /* Choice */
  fltk::Choice *primitivemenu = new fltk::Choice(210, 10, 100, 20, "Choose type of primitive to add:");
  //fltk::MenuBar *primitivemenu = new fltk::MenuBar(5, 2, 600, 30); 
  //  primitivemenu->box(fltk::PLASTIC_UP_BOX);
  //  subwindow = new fltk::Group(10, 50, 600, 340, "Insert primitive data here:");
  subwindow = new fltk::Group(10, 50, 600, 340, "");
  subwindow->box(fltk::SHADOW_BOX);

  /* Filename */
  fltk::Input *filenameinput = new fltk::Input(110, 400, 250, 25, "Filename :");
  //  filenameinput->box(fltk::SHADOW_BOX);
  filenameinput_i = boundarygroup->find(filenameinput);

  /* Filename browser button */
  fltk::Button *filenamebrowser = new fltk::Button(362, 400, 25, 25, "");
  filenamebrowser->callback((fltk::Callback *)openfilebrowser);
  filenamebrowser->image((fltk::Image *)open_icon);
  filenamebrowser->tooltip("Browse");
  //  filenamebrowser->buttonbox(fltk::SHADOW_BOX);

  /* Create file button */
  fltk::Button *createfilebutton = new fltk::Button(390, 400, 100, 25, "Create");
  createfilebutton->callback((fltk::Callback *)createfile);
  createfilebutton->image((fltk::Image *)create_icon);
  createfilebutton->tooltip("Create the output boundary file");
  //  createfilebutton->buttoncolor(fltk::GRAY20);
  //  createfilebutton->box(fltk::PLASTIC_UP_BOX);
  //  createfilebutton->buttonbox(fltk::PLASTIC_UP_BOX);

  /* PRIMITIVE MENU SETUP */
  /*  this is inside boundary tab */
  primitivemenu->add("&clear", 0, (fltk::Callback *)clear_cb, 0, 0);
  primitivemenu->add("&line", 0, (fltk::Callback *)line_cb, 0, 0);
  primitivemenu->add("&circle", 0, (fltk::Callback *)circle_cb, 0, 0);
  primitivemenu->add("&ellipse", 0, (fltk::Callback *)ellipse_cb, 0, 0);
  primitivemenu->add("&data", 0, (fltk::Callback *)clear_cb, 0, 0);

  /*
  primitivemenu->begin();
  lineitem = new fltk::Item("Line");
  lineitem->callback((fltk::Callback*)line_cb);
  lineitem->image(fltk::pngImage::get("line_1.png"));
  circleitem = new fltk::Item("Circle");
  circleitem->callback((fltk::Callback*)circle_cb);
  circleitem->image(fltk::pngImage::get("circle_1.png"));
  ellipseitem = new fltk::Item("Ellipse");
  ellipseitem->callback((fltk::Callback*)ellipse_cb);
  ellipseitem->image(fltk::pngImage::get("ellipse_1.png"));
  dataitem = new fltk::Item("Data");
  dataitem->callback((fltk::Callback*)data_cb);
  dataitem->image(fltk::pngImage::get("data_1.png"));
  primitivemenu->end();
  */

  /* OPENGL WINDOW, inside boundary tab */
  //  u2d_gl = new u2d_GL::u2d_GL(460, 5, 70, 70, 0);
  //  u2d_gl = new u2d_GL::u2d_GL(10, 150, 150, 150, 0);

  boundarygroup->end();

  /* DOMAIN TAB */
  domaingroup->begin();
  /* PLACEHOLDER */
  domaingroup->end();

  /* MENUBAR SETUP */
  menubar->begin();
  /* file menu */
  fltk::ItemGroup *filemenu = new fltk::ItemGroup("&File");
  filemenu->begin();
  fltk::Item *openitem = new fltk::Item("&Open");
  openitem->callback((fltk::Callback *)openitem_cb);
  fltk::Item *saveitem = new fltk::Item("&Save");
  saveitem->callback((fltk::Callback *)saveitem_cb);
  saveitem->image((fltk::Image *)save_icon);
  //  cancel->tooltip("Cancel");
  fltk::Item *saveasitem = new fltk::Item("&Save As...");
  saveasitem->callback((fltk::Callback *)saveasitem_cb);
  fltk::Item *quititem = new fltk::Item("&Quit");
  quititem->callback((fltk::Callback *)quititem_cb);
  filemenu->end();
  /* view menu */
  fltk::ItemGroup *view = new fltk::ItemGroup("&View");
  view->begin();
  fltk::Item *glview = new fltk::Item("&Toggle openGL preview");
  glview->callback((fltk::Callback *)glview_cb);
  fltk::Item *browserview = new fltk::Item("&Toggle Primirive preview");
  browserview->callback((fltk::Callback *)browserview_cb);
  view->end();
  /* tools menu */
  fltk::ItemGroup *toolsmenu = new fltk::ItemGroup("&Tools");
  toolsmenu->begin();
  fltk::Item *main_coreitem = new fltk::Item("&Evaluate closed objects");
  main_coreitem->callback((fltk::Callback *)main_core_cb);
  toolsmenu->end();
  /* help menu */
  fltk::ItemGroup *help = new fltk::ItemGroup("&Help");
  help->begin();
  fltk::Item *about = new fltk::Item("&About umesh2dgui");
  about->callback((fltk::Callback *)about_cb);

#ifdef DEBUGGING  
  fltk::Item *debugitem = new fltk::Item("&Debug callback");
  debugitem->callback((fltk::Callback *)debug_cb);
#endif
  
  help->end();
  menubar->end();

  /* RESIZABLE SETUP */
  prim_browser_window->resizable(prim_browser_window);

  /* LOOP*/
  window->show(argc, argv);
  subwindow->show();

  opengl_window->show();

  prim_browser_window->show();

  int val;
  val = fltk::run();

  /* clear the childs */
  window->clear();
  opengl_window->clear();
  prim_browser_window->clear();

  /* deinstantiate the parents */
  delete window;
  delete opengl_window;
  delete prim_browser_window;
  /* delete the icons */
  delete open_icon;
  delete save_icon;
  delete create_icon;
  delete cancel_icon;
  /* delete our classes */
  delete bdfile;
  delete u2d_core;

  return val;

}

void openfilebrowser() {
  ((fltk::Input *)boundarygroup->child(filenameinput_i))->value(fltk::file_chooser("Select a file", NULL, NULL, true)); 
  /* try to call file browser several time. Value() wants a good static buffer or what?
   * LOOK LIKE WORKING... but i'm really not sure if will under extreme condition
   */
}

void createfile() {
  int returnvalue;
  if(strlen(((fltk::Input *)boundarygroup->child(filenameinput_i))->value()) < 1){
    printf("Filename is empty. Opening file browser\n");
    openfilebrowser();
  }
  if(strlen(((fltk::Input *)boundarygroup->child(filenameinput_i))->value()) < 1){
    printf("Filename is empty. User aborted\n");
    return;
  }
  strncpy(bdfile->bfilename,((fltk::Input *)boundarygroup->child(filenameinput_i))->value(), 256);
  strncpy(bdfile->dfilename,((fltk::Input *)boundarygroup->child(filenameinput_i))->value(), 256);

  /* Calculate closed objects */
  fprintf(stdout, "Calculating closed objects and vertex enumeration...\n");
  u2d_core->find_objects();
  fprintf(stdout, "Done!\n");

  /* Initializate the BDFile class with necessary
   * parameters */
  bdfile->select_prim_list(u2d_core->get_prim_list_start(), u2d_core->get_prim_list_end(), \
			   u2d_core->get_prim_number(), u2d_core->get_vertex_n());

  /* Call the output boundary file function of the
   * selected class for output */
  returnvalue = bdfile->outputbfile();
  if(returnvalue < 0) {
    fprintf(stderr, "createfile: Error creating the boundary file\n");
    if(returnvalue == -1) fltk::alert("Error creating boundary file: Invalid filename\n");
    else if(returnvalue == -2) fltk::alert("Error creating boundary file: File opening was not possible\n");
    else if(returnvalue == -3) fltk::alert("Notting to output or list initialization error\n");
    else fltk::alert("Error creating boundary file: Unkown reason\n");
    return;
  }
  returnvalue = bdfile->outputdfile();
  if(returnvalue < 0) {
    fprintf(stderr, "createfile: Error creating the domain file\n");
    if(returnvalue == -1) fltk::alert("Error creating domain file: Invalid filename\n");
    else if(returnvalue == -2) fltk::alert("Error creating domain file: File opening was not possible\n");
    else if(returnvalue == -3) fltk::alert("Notting to output or list initialization error\n");
    else fltk::alert("Error creating domain file: Unkown reason\n");
    return;
  }
  fprintf(stdout, "File creation sucessfull\n");
  msgout->value("File creation sucessfull");
}

void line_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  //lineitem->box(fltk::DOWN_BOX);
  subwindow->begin();

  /* spline*/
  fltk::ValueInput *splinepoints = new fltk::ValueInput(200, 10, 100, 20, "N. points for spline aprox.: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  splinepoints->set_value(100);
  //  splinepoints->pagesize(10.0);
  inputindex[0] = subwindow->find(splinepoints);
  //  printf("%i", inputindex[0]);

  /* name */
  fltk::Input *primname = new fltk::Input(450, 10, 100, 20, "Name (Optional): ");
  inputindex[1] = subwindow->find(primname);

  /* x1 */
  //  fltk::ValueInput *x1 = new fltk::ValueInput(200, 30, 100, 20, "Start point x coordinate: ");
  fltk::ValueInput *x1 = new fltk::ValueInput(200, 30, 100, 20, "Start point x: ");
  x1->range(-1000, 1000);
  x1->step(0.01);
  x1->linesize(0.01);
  //  x1->pagesize(1.0);
  inputindex[2] = subwindow->find(x1);
  //  printf("%i",  inputindex[1]);

  /* y1 */
  fltk::ValueInput *y1 = new fltk::ValueInput(450, 30, 100, 20, "Start point y: ");
  y1->range(-1000, 1000);
  y1->step(0.01);
  y1->linesize(0.01);
  //  y1->pagesize(1.0);
  inputindex[3] = subwindow->find(y1);
  //  printf("%i", inputindex[2]);

  /* x2 */
  fltk::ValueInput *x2 = new fltk::ValueInput(200, 50, 100, 20, "End point x: ");
  x2->range(-1000, 1000);
  x2->step(0.01);
  x2->linesize(0.01);
  //  x2->pagesize(1.0);
  inputindex[4] = subwindow->find(x2);
  //  printf("%i", inputindex[3]);

  /* y2 */
  fltk::ValueInput *y2 = new fltk::ValueInput(450, 50, 100, 20, "End point y: ");
  y2->range(-1000, 1000);
  y2->step(0.01);
  y2->linesize(0.01);
  //  y2->pagesize(1.0);
  inputindex[5] = subwindow->find(y2);
  //  printf("%i", inputindex[4]);
  /* boundary */
  //  fltk::CheckButton *isboundary = new fltk::CheckButton(200, 70, 100, 20, "Boundary: ");
  //  isboundary->align(fltk::ALIGN_LEFT);
  //  inputindex[6] = subwindow->find(isboundary);
  /* Datagroup for [i|s|h]data */
  /*  fltk::Group *databox = new fltk::Group(10, 130, 580, 160, "");
  databox->box(fltk::BORDER_FRAME);
  */
  fltk::Button *new_dataline = new fltk::Button(10, 100, 580, 20, "Add data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  //  datagroup = new fltk::ScrollGroup(10, 140, 580, 160, "Insert [i|s|h]data here:");
  datagroup = new fltk::ScrollGroup(10, 130, 580, 160, "");
  datagroup->scrollbar.linesize(20);
  new_dataline_cb();
  new_dataline_cb();
  ((fltk::ValueInput *)(datagroup->child(1)))->set_value(0);
  ((fltk::ValueInput *)(datagroup->child(2)))->set_value(0.05);
  ((fltk::ValueInput *)(datagroup->child(4)))->set_value(1);
  ((fltk::ValueInput *)(datagroup->child(5)))->set_value(0.05);
  //  datagroup->box(fltk::BORDER_FRAME);
  //  datagroup->begin();
  //  fltk::Widget *box2 = new fltk::Widget(0, 0, 620, 160, "TEXT"); // this is to see the space avaiable
  //  fltk::Scroll *datascroll = new fltk::Scroll(600, 0, 20, 160, 0);
  //  datagroup->end();

  /* Buttons (Add, Cancel)*/
  fltk::Button *cancel = new fltk::Button(200, 305, 100, 25, "Cancel");
  cancel->image((fltk::Image *)cancel_icon);
  cancel->tooltip("Cancel");
  fltk::Button *add = new fltk::Button(320, 305, 100, 25, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_line_cb);
  subwindow->end();
  subwindow->redraw();

  /* CHECKING AND FLUSHING */
#ifdef DEBUGGING
  printf("Selected line primitive\n");
  fflush(stdout);
#endif
}

void circle_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  //circleitem->box(fltk::DOWN_BOX);
  subwindow->begin();
  /* spline */
  fltk::ValueInput *splinepoints = new fltk::ValueInput(200, 10, 100, 20, "N. points for spline aprox.: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  splinepoints->set_value(100);
  inputindex[0] = subwindow->find(splinepoints);
  /* name */
  fltk::Input *primname = new fltk::Input(450, 10, 100, 20, "Name (Optional): ");
  inputindex[1] = subwindow->find(primname);
  /* radius */
  fltk::ValueInput *radius = new fltk::ValueInput(200, 30, 100, 20, "Radius: ");
  radius->range(-1000, 1000);
  radius->step(0.01);
  inputindex[2] = subwindow->find(radius);
  /* center x */
  fltk::ValueInput *centerx = new fltk::ValueInput(200, 50, 100, 20, "Center abscissa (x): ");
  centerx->range(-1000, 1000);
  centerx->step(0.01);
  inputindex[3] = subwindow->find(centerx);
  /* center y */
  fltk::ValueInput *centery = new fltk::ValueInput(450, 50, 100, 20, "Center ordinata (y): ");
  centery->range(-1000, 1000);
  centery->step(0.01);
  inputindex[4] = subwindow->find(centery);
  /* start arc */
  fltk::ValueInput *sarc = new fltk::ValueInput(200, 70, 100, 20, "Start angle (deg): ");
  sarc->range(-360, 360);
  sarc->linesize(1.0);
  sarc->step(0.1);
  inputindex[5] = subwindow->find(sarc);
  /* end arc */
  fltk::ValueInput *earc = new fltk::ValueInput(450, 70, 100, 20, "End angle (deg): ");
  earc->range(-360, 360);
  earc->linesize(1.0);
  earc->step(0.1);
  inputindex[6] = subwindow->find(earc);
  /* is boundary */
  //  fltk::CheckButton *isboundary = new fltk::CheckButton(200, 90, 100, 20, "Boundary: ");
  //  isboundary->align(fltk::ALIGN_LEFT);
  //  inputindex[7] = subwindow->find(isboundary);
  /* [i|s|h]data */
  fltk::Button *new_dataline = new fltk::Button(10, 120, 580, 20, "Add data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  datagroup = new fltk::ScrollGroup(10, 150, 580, 140, "");
  datagroup->scrollbar.linesize(20);
  new_dataline_cb();
  new_dataline_cb();
  ((fltk::ValueInput *)(datagroup->child(1)))->set_value(0);
  ((fltk::ValueInput *)(datagroup->child(2)))->set_value(0.05);
  ((fltk::ValueInput *)(datagroup->child(4)))->set_value(1);
  ((fltk::ValueInput *)(datagroup->child(5)))->set_value(0.05);
  /* buttons */
  fltk::Button *cancel = new fltk::Button(200, 305, 100, 25, "Cancel");
  cancel->image((fltk::Image *)cancel_icon);
  cancel->tooltip("Cancel");
  fltk::Button *add = new fltk::Button(320, 305, 100, 25, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_circle_cb);
  subwindow->end();
  subwindow->redraw();
  //  subwindow->redraw(0, 0, 620, 380);

  /* CHECKING AND FLUSHING */
#ifdef DEBUGGING
  printf("Selected circle primitive\n");
  fflush(stdout);
#endif
}

void ellipse_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  //ellipseitem->box(fltk::DOWN_BOX);
  subwindow->begin();
  /* spline */
  fltk::ValueInput *splinepoints = new fltk::ValueInput(200, 10, 100, 20, "N. points for spline aprox.: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  splinepoints->set_value(100);
  inputindex[0] = subwindow->find(splinepoints);
  /* name */
  fltk::Input *primname = new fltk::Input(450, 10, 100, 20, "Name (Optional): ");
  inputindex[1] = subwindow->find(primname);
  /* semix */
  fltk::ValueInput *semix = new fltk::ValueInput(200, 30, 100, 20, "x semiaxis length: ");
  semix->range(-1000, 1000);
  semix->step(0.01);
  inputindex[2] = subwindow->find(semix);
  /* semiy */
  fltk::ValueInput *semiy = new fltk::ValueInput(450, 30, 100, 20, "y semiaxis length: ");
  semiy->range(-1000, 1000);
  semiy->step(0.01);
  inputindex[3] = subwindow->find(semiy);
  /* center x */
  fltk::ValueInput *centerx = new fltk::ValueInput(200, 50, 100, 20, "Center abscissa (x): ");
  centerx->range(-1000, 1000);
  centerx->step(0.01);
  inputindex[4] = subwindow->find(centerx);
  /* center y */
  fltk::ValueInput *centery = new fltk::ValueInput(450, 50, 100, 20, "Center ordinata (y): ");
  centery->range(-1000, 1000);
  centery->step(0.01);
  inputindex[5] = subwindow->find(centery);
  /* start arc */
  fltk::ValueInput *sarc = new fltk::ValueInput(200, 70, 100, 20, "Start angle (deg): ");
  sarc->range(-360, 360);
  sarc->linesize(1.0);
  sarc->step(0.1);
  inputindex[6] = subwindow->find(sarc);
  /* end arc */
  fltk::ValueInput *earc = new fltk::ValueInput(450, 70, 100, 20, "End angle (deg): ");
  earc->range(-360, 360);
  earc->linesize(1.0);
  earc->step(0.1);
  inputindex[7] = subwindow->find(earc);
  /* is boundary */
  //  fltk::CheckButton *isboundary = new fltk::CheckButton(200, 90, 100, 20, "Boundary: ");
  //  isboundary->align(fltk::ALIGN_LEFT);
  //  inputindex[8] = subwindow->find(isboundary);
  /* [i|s|h]data */
  fltk::Button *new_dataline = new fltk::Button(10, 120, 580, 20, "Add data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  datagroup = new fltk::ScrollGroup(10, 150, 580, 140, "");
  datagroup->scrollbar.linesize(20);
  new_dataline_cb();
  new_dataline_cb();
  ((fltk::ValueInput *)(datagroup->child(1)))->set_value(0);
  ((fltk::ValueInput *)(datagroup->child(2)))->set_value(0.05);
  ((fltk::ValueInput *)(datagroup->child(4)))->set_value(1);
  ((fltk::ValueInput *)(datagroup->child(5)))->set_value(0.05);
  /* buttons */
  fltk::Button *cancel = new fltk::Button(200, 305, 100, 25, "Cancel");
  cancel->image((fltk::Image *)cancel_icon);
  cancel->tooltip("Cancel");
  fltk::Button *add = new fltk::Button(320, 305, 100, 25, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_ellipse_cb);
  subwindow->end();
  subwindow->redraw();
  //  subwindow->redraw(0, 0, 620, 380);

  /* CHECKING AND FLUSHING */
#ifdef DEBUGGING
  printf("Selected ellipse primitive\n");
  fflush(stdout);
#endif
}

void new_dataline_cb() {
  int currentpos;
  datagroup->begin();
  /* This is a hard hack since inputs (and any other thing) got created 
   * from actual origin and not from widget (datagroup inn this case)
   * absolute position, anyway the first time the input got creted with
   * the bar fully scored and unexpected 10 pixel offset is generated.
   * I suspect this is an initialization bug in fltk code.
   * I figured out the problem and resoleved it, it is a bad, but not
   * buggy, behavor or fltk. It just wipesout white space if not filled
   * by some widget (window or anything) */
  if(datagroup->yposition() < 0) {
    currentpos = ydataposition;
  }
  else {
    currentpos = ydataposition - datagroup->yposition();
  }
  //  fprintf(stdout, "yposition(): %i\n", datagroup->yposition());
  //  fprintf(stdout, "currentpos: %i\n", currentpos);
  //  fflush(stdout);
  //  ydataposition = ydataposition - datagroup->yposition();
  fltk::ValueInput *idata = new fltk::ValueInput(50, currentpos, 100, 20, "idata: ");
  idata->range(0, 3);
  idata->linesize(1.0);
  idata->set_value(1);
  fltk::ValueInput *sdata = new fltk::ValueInput(250, currentpos, 100, 20, "sdata: ");
  sdata->range(0, 1);
  sdata->linesize(0.01);
  sdata->set_value(1);
  fltk::ValueInput *hdata = new fltk::ValueInput(450, currentpos, 100, 20, "hdata: ");
  hdata->range(0, 1);
  hdata->linesize(0.01);
  hdata->set_value(0.05);
  datagroup->end();
  datagroup->redraw();
  //  datagroup->redraw(0, currentpos, 620, 20);  // maybe this is not anymore necessary, but should be correct.
  ydataposition = ydataposition + 20;
}

void clear_cb(){
  /* reset the scroll position */
  ydataposition = 0;
  /* clean the index for primitive values */
  for(int i = 0; i < 10; i++) inputindex[i] = 0;
  /* reset button state */
  //lineitem->box(fltk::NO_BOX);
  //circleitem->box(fltk::NO_BOX);
  //ellipseitem->box(fltk::NO_BOX);
  //dataitem->box(fltk::NO_BOX);
  /* clear teh window */
  subwindow->clear();
  subwindow->redraw();
  printf("clear done\n");
  //  subwindow->redraw(0, 0, 620, 330);
}

void add_line_cb() {
  int ndata;
  int validdata = 0;
  Line *prim = (Line *)(u2d_core->add_prim(PR_LINE));
  //  printf("subwindow ha %i figli\n", subwindow->children());
  //  strncpy(prim->primname, "ciao", 31);
  strncpy(prim->primname, ((fltk::Input *)subwindow->child(inputindex[1]))->value(), 32);
  /* Changed to use the new function
  prim->spline = (int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value();
  prim->x1 = ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value();
  prim->y1 = ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value();
  prim->x2 = ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value();
  prim->y2 = ((fltk::ValueInput *)subwindow->child(inputindex[5]))->value();
  */
  prim->set_coord((int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value(),\
                  ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value(),\
		  ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value(),\
		  ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value(),\
		  ((fltk::ValueInput *)subwindow->child(inputindex[5]))->value());
  printf("Spline: %i x1: %f y1: %f x2: %f y2: %f\n", prim->spline, prim->x1, prim->y1, prim->x2, prim->y2);
  fflush(stdout);
  //  prim->isboundary = ((fltk::CheckButton *)subwindow->child(inputindex[6]))->value();
  //  printf("Boundary : %s\n",  (prim->isboundary) ? "true" : "false");
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* this could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
  add_generic(prim);
}

void add_circle_cb() {
  int ndata;
  int validdata = 0;
  Circle *prim = (Circle *)(u2d_core->add_prim(PR_CIRCLE));
#ifdef DEBUGGING
  printf("subwindow ha %i figli\n", subwindow->children());
#endif
  strncpy(prim->primname, ((fltk::Input *)subwindow->child(inputindex[1]))->value(), 32);
  /* Changed to use the new function
  prim->spline = (int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value();
  prim->radius = ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value();
  prim->centerx = ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value();
  prim->centery = ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value();
  prim->startarc = ((fltk::ValueInput *)subwindow->child(inputindex[5]))->value();
  prim->endarc = ((fltk::ValueInput *)subwindow->child(inputindex[6]))->value();
  */
  prim->set_coord((int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value(),/* spline */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value(), /* radius */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value(), /* centerx */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value(), /* centery */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[5]))->value(), /* startarc */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[6]))->value()); /* endarc */
  printf("Spline: %i radius: %f centerx: %f centery: %f startarc: %f endarc: %f\n", prim->spline, prim->radius, prim->centerx, prim->centery, prim->startarc, prim->endarc);
  fflush(stdout);
  //  prim->isboundary = ((fltk::CheckButton *)subwindow->child(inputindex[7]))->value();
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* This for cycle could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
  add_generic(prim);
}

void add_ellipse_cb() {
  int ndata;
  int validdata = 0;
  Ellipse *prim = (Ellipse *)(u2d_core->add_prim(PR_ELLIPSE));
#ifdef DEBUGGING
  printf("subwindow ha %i figli\n", subwindow->children());
#endif
  strncpy(prim->primname, ((fltk::Input *)subwindow->child(inputindex[1]))->value(), 32);
  /* Changed to use the new function
  prim->spline = (int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value();
  prim->semix = ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value();
  prim->semiy = ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value();
  prim->centerx = ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value();
  prim->centery = ((fltk::ValueInput *)subwindow->child(inputindex[5]))->value();
  prim->startarc = ((fltk::ValueInput *)subwindow->child(inputindex[6]))->value();
  prim->endarc = ((fltk::ValueInput *)subwindow->child(inputindex[7]))->value();
  */
  prim->set_coord((int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value(),  /* spline */ \
  	  ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value(), /* semix */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value(), /* semiy */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value(), /* centerx */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[5]))->value(), /* centery */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[6]))->value(), /* startarc */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[7]))->value()); /* endarc */
  printf("Spline: %i semix: %f semiy: %f centerx: %f centery: %f startarc: %f endarc: %f\n", prim->spline, prim->semix, prim->semiy, prim->centerx, prim->centery, prim->startarc, prim->endarc);
  fflush(stdout);
  //  prim->isboundary = ((fltk::CheckButton *)subwindow->child(inputindex[8]))->value();
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* This could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
  add_generic(prim);
}

void add_generic(Primitive *prim){
#ifdef DEBUGGING
  fprintf(stdout, "Calling setprim\n");
#endif
  u2d_gl->set_prim_list_start(u2d_core->get_prim_list_start(), \
		              u2d_core->get_prim_number());
  u2d_gl->guess_ref_size(prim);
  u2d_gl->redraw();
  prim_browser->add(u2d_core->get_prim_string(prim), 0, NULL, 0, 0);
}

void openitem_cb(){
 fltk::message("Open Feature not avaialbe yet\n");
}

void saveasitem_cb(){
  openfilebrowser();
  createfile();
}

void saveitem_cb(){
  if(strlen(((fltk::Input *)boundarygroup->child(filenameinput_i))->value()) < 1) saveasitem_cb();
  else createfile();
}

void quititem_cb(){
  window->hide();
  opengl_window->hide();
  prim_browser_window->hide();
  /* calling the upper 2 or this one is exacly the same, no idea what is better */
  /* exit(0); */
}

void about_cb(){
  fltk::message("%s by Gabriele Dini Ciacci - version %s\n", PACKAGE, VERSION);
}

void glview_cb(){
  if(opengl_window->visible()){
    opengl_window->hide();
  }
  else {
    opengl_window->show();
  }
  /* Removed because now we hide the main detached window
   * and not anymore the gl window itself */
  /*
  if(u2d_gl->visible()){
    u2d_gl->hide();
  }
  else {
    u2d_gl->show();
  }
  */
}

void browserview_cb(){
  if(prim_browser_window->visible()){
    prim_browser_window->hide();
  }
  else {
    prim_browser_window->show();
  }
}

void browser_del_prim_cb() {
  fltk::Widget* current = prim_browser->goto_top();
  while (current) {
    if (current->selected()) {
      /* This work in case of hierarical menus too.
       * In my case the current->parent() should allways be
       * prim_browser */
      fltk::Group* parent = current->parent();
      parent->remove(current);
      /* Now we  delete the primitive from the list like:
       * id = current->value() to get the removed primitive id
       * del_prim(get_prim_by_id(id)); */
      u2d_core->del_prim(u2d_core->get_prim_by_label(current->label()));
      delete current;
      parent->relayout();
      /* We start again from top to see if there are
       * still other primitives selected */
      current = prim_browser->goto_top();
    }
    else {
      current = prim_browser->next();
    }
  }

  /* We have to update the openGL window too */
  u2d_gl->set_prim_list_start(u2d_core->get_prim_list_start(), \
			      u2d_core->get_prim_number());
  /* We have to recalculate the reference sizes from
   * scratch... VERY costly */
  u2d_gl->guess_ref_size();
  /* We can finally redaw */
  u2d_gl->redraw();
}

void main_core_cb(){
  /* Calculate closed objects */
  fprintf(stdout, "Calculating closed objects and vertex enumeration...\n");
  u2d_core->find_objects();
  fprintf(stdout, "Done!\n");
}

void data_cb(){
  clear_cb();
  //dataitem->box(fltk::DOWN_BOX);
  subwindow->begin();

  /* rotation */
  fltk::ValueInput *rotation = new fltk::ValueInput(190, 10, 100, 20, "Spline aprox. points: ");
  rotation->range(-360, 360);
  rotation->linesize(1.0);
  rotation->step(1);
  rotation->set_value(0);
  inputindex[0] = subwindow->find(rotation);

  /* name */
  fltk::Input *primname = new fltk::Input(450, 10, 100, 20, "Name (Optional): ");
  inputindex[1] = subwindow->find(primname);


  /* start point bordered box */
  fltk::Group *scontour  = new fltk::Group(50, 30, 240, 44, "Displacement ");
  scontour->box(fltk::BORDER_FRAME);
  scontour->align(fltk::ALIGN_INSIDE_LEFT);

  /* trasx */
  fltk::ValueInput *trasx = new fltk::ValueInput(190, 32, 98, 20, "x: ");
  trasx->range(-1000, 1000);
  trasx->step(0.01);
  trasx->linesize(0.01);
  inputindex[2] = subwindow->find(trasx);

  /* trasy */
  fltk::ValueInput *trasy = new fltk::ValueInput(190, 52, 98, 20, "y: ");
  trasy->range(-1000, 1000);
  trasy->step(0.01);
  trasy->linesize(0.01);
  inputindex[3] = subwindow->find(trasy);

  /* data filename */
  fltk::Input *datafilename = new fltk::Input(COLUMN2, 74, 250, 20, "Filename :");
  inputindex[4] = subwindow->find(datafilename);

  /* Datagroup for [i|s|h]data */
  /*  fltk::Group *databox = new fltk::Group(10, 130, 580, 160, "");
  databox->box(fltk::BORDER_FRAME);
  */
  fltk::Button *new_dataline = new fltk::Button(10, 100, 580, 20, "Add data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  datagroup = new fltk::ScrollGroup(10, 130, 580, 160, "");
  datagroup->scrollbar.linesize(20);
  new_dataline_cb();
  new_dataline_cb();
  ((fltk::ValueInput *)(datagroup->child(1)))->set_value(0);
  ((fltk::ValueInput *)(datagroup->child(2)))->set_value(0.05);
  ((fltk::ValueInput *)(datagroup->child(4)))->set_value(1);
  ((fltk::ValueInput *)(datagroup->child(5)))->set_value(0.05);
  //  datagroup->box(fltk::BORDER_FRAME);
  //  datagroup->begin();
  //  fltk::Widget *box2 = new fltk::Widget(0, 0, 620, 160, "TEXT"); // this is to see the space avaiable
  //  fltk::Scroll *datascroll = new fltk::Scroll(600, 0, 20, 160, 0);
  //  datagroup->end();

  /* Buttons (Add, Cancel)*/
  fltk::Button *cancel = new fltk::Button(200, 305, 100, 25, "Cancel");
  cancel->image((fltk::Image *)cancel_icon);
  cancel->tooltip("Cancel");
  fltk::Button *add = new fltk::Button(320, 305, 100, 25, "Load");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_data_cb);
  subwindow->end();
  subwindow->redraw();

  /* CHECKING AND FLUSHING */
#ifdef DEBUGGING
  printf("Selected data\n");
  fflush(stdout);
#endif

}


void add_data_cb(){
  int ndata;
  int validdata = 0;
  int retval;
  Data *prim = (Data *)(u2d_core->add_prim(PR_DATA));
#ifdef DEBUGGING
  printf("subwindow ha %i figli\n", subwindow->children());
#endif
  strncpy(prim->primname, ((fltk::Input *)subwindow->child(inputindex[1]))->value(), 32);
  retval = prim->set_coord(((fltk::Input *)subwindow->child(inputindex[4]))->value(),  /* filename */ \
  	  ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value(), /* trasx */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value(), /* trasy */ \
    	  ((fltk::ValueInput *)subwindow->child(inputindex[0]))->value()); /* rotation */
  if(retval < 0) {
    if(retval == NO_FILE) fltk::alert("Error opening file: Invalid data filename\n");
    else if(retval == INVALID_FILE) fltk::alert("Error reading file: check file syntax\n");
    else fltk::alert("Error opening file: Unkown reason\n");
    u2d_core->del_prim(prim);
    return;
  }
   printf("Filename: %s trasx: %f trasy: %f rotation: %f\n",\
	  prim->filename, prim->trasx, prim->trasy,\
	  prim->rotation);
  fflush(stdout);
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* This could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
  add_generic(prim);

}

void debug_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  subwindow->begin();

  /* spline*/
  fltk::ValueInput *splinepoints = new fltk::ValueInput(190, 10, 100, 20, "Spline aprox. points: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  splinepoints->set_value(100);
  //  splinepoints->pagesize(10.0);
  inputindex[0] = subwindow->find(splinepoints);
  //  printf("%i", inputindex[0]);

  /* name */
  fltk::Input *primname = new fltk::Input(450, 10, 100, 20, "Name (Optional): ");
  inputindex[1] = subwindow->find(primname);

  /* start point bordered box */
  fltk::Group *scontour  = new fltk::Group(50, 30, 240, 44, "Start point");
  scontour->box(fltk::BORDER_FRAME);
  scontour->align(fltk::ALIGN_INSIDE_LEFT);

  /* x1 */
  fltk::ValueInput *x1 = new fltk::ValueInput(190, 32, 98, 20, "x: ");
  x1->range(-1000, 1000);
  x1->step(0.01);
  x1->linesize(0.01);
  //  x1->pagesize(1.0);
  inputindex[2] = subwindow->find(x1);
  //  printf("%i",  inputindex[1]);

  /* y1 */
  fltk::ValueInput *y1 = new fltk::ValueInput(190, 52, 98, 20, "y: ");
  y1->range(-1000, 1000);
  y1->step(0.01);
  y1->linesize(0.01);
  //  y1->pagesize(1.0);
  inputindex[3] = subwindow->find(y1);
  //  printf("%i", inputindex[2]);

  /* end point bordered box */
  fltk::Group *econtour  = new fltk::Group(310, 30, 240, 44, "End point");
  econtour->box(fltk::BORDER_FRAME);
  econtour->align(fltk::ALIGN_INSIDE_LEFT);

  /* x2 */
  fltk::ValueInput *x2 = new fltk::ValueInput(450, 32, 98, 20, "x: ");
  x2->range(-1000, 1000);
  x2->step(0.01);
  x2->linesize(0.01);
  //  x2->pagesize(1.0);
  inputindex[4] = subwindow->find(x2);
  //  printf("%i", inputindex[3]);

  /* y2 */
  fltk::ValueInput *y2 = new fltk::ValueInput(450, 52, 98, 20, "y: ");
  y2->range(-1000, 1000);
  y2->step(0.01);
  y2->linesize(0.01);
  //  y2->pagesize(1.0);
  inputindex[5] = subwindow->find(y2);
  /*
  fltk::CheckButton *isboundary = new fltk::CheckButton(200, 70, 100, 20, "Boundary: ");
  isboundary->align(fltk::ALIGN_LEFT);
  inputindex[6] = subwindow->find(isboundary);
  */
  /* Datagroup for [i|s|h]data */
  /*  fltk::Group *databox = new fltk::Group(10, 130, 580, 160, "");
  databox->box(fltk::BORDER_FRAME);
  */
  fltk::Button *new_dataline = new fltk::Button(10, 100, 580, 20, "Press here to add a new data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  //  datagroup = new fltk::ScrollGroup(10, 140, 580, 160, "Insert [i|s|h]data here:");
  datagroup = new fltk::ScrollGroup(10, 130, 580, 160, "");
  datagroup->scrollbar.linesize(20);
  new_dataline_cb();
  new_dataline_cb();
  ((fltk::ValueInput *)(datagroup->child(1)))->set_value(0);
  ((fltk::ValueInput *)(datagroup->child(2)))->set_value(0.05);
  ((fltk::ValueInput *)(datagroup->child(4)))->set_value(1);
  ((fltk::ValueInput *)(datagroup->child(5)))->set_value(0.05);
  //  datagroup->box(fltk::BORDER_FRAME);
  //  datagroup->begin();
  //  fltk::Widget *box2 = new fltk::Widget(0, 0, 620, 160, "TEXT"); // this is to see the space avaiable
  //  fltk::Scroll *datascroll = new fltk::Scroll(600, 0, 20, 160, 0);
  //  datagroup->end();

  /* Buttons (Add, Cancel)*/
  fltk::Button *cancel = new fltk::Button(200, 305, 100, 25, "Cancel");
  cancel->image((fltk::Image *)cancel_icon);
  cancel->tooltip("Cancel");
  fltk::Button *add = new fltk::Button(320, 305, 100, 25, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_line_cb);
  subwindow->end();
  subwindow->redraw();

  /* CHECKING AND FLUSHING */
#ifdef DEBUGGING
  printf("Selected debug\n");
  fflush(stdout);
#endif


  //  u2d_core->debug_cb();
}

